
int  AdoDBErrorNumber;
wchar_t AdoDBErrorDescription[1024];

int  CursorErrorNumber;
wchar_t CursorErrorDescription[1024];


HWND hCurrencyList = NULL;
wchar_t bufferW[32768];
//int  IndexTrigger;
//int  CursorTrigger;


// database definition
#define MAX_CONNECTIONS 256
struct CONNECTION
      {
		int     Id;
        _ConnectionPtr Db;
        int  AdoDBErrorNumber;
        wchar_t AdoDBErrorDescription[1024];
      };

CONNECTION Connections[MAX_CONNECTIONS];

// cursor definition
#define MAX_CURSORS 1024
struct CURSOR
      {
		int     Id;
        int     ConnectionID;
		bool    FirstRow;
		int     Fields;
		 _RecordsetPtr RS;
		wchar_t Value[1024];
        int  CursorErrorNumber;
        wchar_t CursorErrorDescription[1024];
      };

CURSOR Cursors[MAX_CURSORS];


// -,     (Windows)
class CAutoMutex
{
  //   
  HANDLE m_h_mutex;

  //  
  CAutoMutex(const CAutoMutex&);
  CAutoMutex& operator=(const CAutoMutex&);
  
public:
  CAutoMutex()
  {
    m_h_mutex = CreateMutex(NULL, FALSE, L"FXCL-ADODB");
    assert(m_h_mutex);
  }
  
  ~CAutoMutex() { CloseHandle(m_h_mutex); }
  
  HANDLE get() { return m_h_mutex; }
};

 // -,    
class CMutexLock
{
  HANDLE m_mutex;

  //  
  CMutexLock(const CMutexLock&);
  CMutexLock& operator=(const CMutexLock&);
public:
  //     
  CMutexLock(HANDLE mutex): m_mutex(mutex)
  {
    const DWORD res = WaitForSingleObject(m_mutex, INFINITE);
    assert(res == WAIT_OBJECT_0);
  }
  //     
  ~CMutexLock()
  {
    const BOOL res = ReleaseMutex(m_mutex);
    assert(res);
  }
};

// ,      
#define SCOPE_LOCK_MUTEX(hMutex) CMutexLock _tmp_mtx_capt(hMutex);

static CAutoMutex g_mutex;

void ClearErrors(int pId)
{
 if (pId == -1)
    {
     AdoDBErrorNumber = 0;
     swprintf(AdoDBErrorDescription, 11, L"No errors.\x00");
    }
 else
    {
     if ((pId>=0) && (pId<MAX_CONNECTIONS))
	    {
         Connections[pId].AdoDBErrorNumber = 0;
         swprintf(Connections[pId].AdoDBErrorDescription, 11, L"No errors.\x00"); 
        }
    }
}

void ClearCursorErrors(int pId)
{
 if (pId == -1)
    {
     CursorErrorNumber = 0;
     swprintf(CursorErrorDescription, 11, L"No errors.\x00");
    }
 else
    {
     if ((pId>=0) && (pId<MAX_CONNECTIONS))
	    {
         Cursors[pId].CursorErrorNumber = 0;
         swprintf(Cursors[pId].CursorErrorDescription, 11, L"No errors.\x00"); 
        }
    }
}

// return free id in connections list
int GetNewConnectionId()
{
	//IndexTrigger++;
	for (int i=0; i<MAX_CONNECTIONS; i++)
		if (Connections[i].Id == -1) return (i);
	return(-1);
}

void DeleteConnection(int pId)
{
	if ((pId>=0) && (pId<MAX_CONNECTIONS))
	{
		//IndexTrigger=0;
		Connections[pId].Id = -1;
		Connections[pId].Db = NULL;
	}
}

// initialization of all connections
void ConnectionsInit()
{
 //  
 SCOPE_LOCK_MUTEX(g_mutex.get());

 ClearErrors(-1);
// IndexTrigger = 0;
 for (int i=0; i<MAX_CONNECTIONS; i++)
     {
	  Connections[i].Id = -1;
	  Connections[i].Db = NULL;
	 }

 if (FAILED(::CoInitialize(NULL)))
    {
     AdoDBErrorNumber = -1;
     swprintf(AdoDBErrorDescription, 32, L"Initialization error.\x00");
	 return;
    }

}

// close all opened connections
void ConnectionsDeinit()
{
	ClearErrors(-1);
//	IndexTrigger = 0;
	for (int i=0; i<MAX_CONNECTIONS; i++)
	{
		if (Connections[i].Id != -1)
		{
         if (Connections[i].Db->State == adStateOpen) 
		    {
		     Connections[i].Db->Close();
	        }
         DeleteConnection(i);
		}
	}
   //::CoUninitialize(); -- mt4 closes by this command
}



// return free id in cursors list
int GetNewCursorId()
{
	//CursorTrigger++;
	for (int i=0; i<MAX_CURSORS; i++)
		if (Cursors[i].Id == -1) return (i);
	return(-1);
}

void DeleteCursor(int pId)
{
	if ((pId>=0) && (pId<MAX_CURSORS))
	{
		//CursorTrigger = 0;
		Cursors[pId].ConnectionID = -1;
		Cursors[pId].Id = -1;
		Cursors[pId].RS = NULL;
		Cursors[pId].FirstRow = true;
		Cursors[pId].Fields = 0;
	}
}

// initialization of all cursors
void CursorsInit()
{
 //CursorTrigger = 0;
 for (int i=0; i<MAX_CURSORS; i++)
     {
      Cursors[i].ConnectionID = -1;
	  Cursors[i].Id = -1;
	  Cursors[i].RS = NULL;
	  Cursors[i].FirstRow = true;
	  Cursors[i].Fields = 0;
	 }
}

// close all opened cursors
void CursorsDeinit()
{
 ClearCursorErrors(-1);
// CursorTrigger = 0;
 for (int i=0; i<MAX_CURSORS; i++)
     {
      if (Cursors[i].Id != -1)
         {
          if (Cursors[i].RS)
             if (Cursors[i].RS->State == adStateOpen)
	            {
                 Cursors[i].RS->Close();
                }
             DeleteCursor(i);
		 }
	}
}

void GenerateGlobalError(int pId)
{
	ErrorPtr pErr = NULL;
	int i;
	int ErrorCount;
	if (pId == -1) return;
	ErrorCount = Connections[pId].Db->Errors->Count;
	if (ErrorCount > 0)
	{
     for (i = 0; i < ErrorCount; i++)
		 {
          pErr = Connections[pId].Db->Errors->GetItem(i);
          swprintf(Connections[pId].AdoDBErrorDescription, 1024, L"%S", (LPCSTR)(pErr->Description));
		  Connections[pId].AdoDBErrorNumber = pErr->NativeError;//			  ->Number;
         }
   }
}

void GenerateCursorError(int pCId)
{
	ErrorPtr pErr = NULL;
	int i;
	int ErrorCount;
	int pId;
	if (pCId == -1) return;
	pId = Cursors[pCId].ConnectionID;
	if (pId == -1) return;
	ErrorCount = Connections[pId].Db->Errors->Count;
	if (ErrorCount > 0)
	{
     for (i = 0; i < ErrorCount; i++)
		 {
          pErr = Connections[pId].Db->Errors->GetItem(i);
		  Cursors[pCId].CursorErrorNumber = pErr->NativeError; //Number;
          swprintf(Cursors[pCId].CursorErrorDescription, 1024, L"%S",(LPCSTR)(pErr->Description));
         }
   }
}